#!/usr/bin/perl
use Math::Trig;
use POSIX;
use List::Util qw(max min);
open OUT, "> out.txt" or die;
print OUT "       cif      mol      d_exp   ro       re    R   N    exp(-2re/R)  error\n";
print "       cif      mol      d_exp   ro       re    R   N    exp(-2re/R)  error\n";
@x,@y,@z,@type,$natom,@void;
open CSV, "< output.csv" or die "ERROR!!! FILE output.csv missing!";
@file=<CSV>;
$len=@file;
for (1 .. $len - 1){
	@x=@y=@z=@type=@void;
	my $line=$file[$_];
	$line=~s/^\s+//;
	$line=~s/\[.*\]\,//g;
	$line=~s/\".*\"\,//g;
	my @line = split /\,/, $line;
	my $name=$line[0];
	my $mol=$line[1];
	my $pol=$line[2];
	my $ro=$line[9];
	my $mu_sigma2=$line[10];
	my @name=split/\./, $name;
	my $cif=shift @name;
	my $flag=&chkerror("$cif");
	next if $flag;
	open LOG, "< $cif.log"; #"****Warrning!!! $cif.log missing\n";
	open CIF, "< $cif.cif"; #"****Warrning!!! $cif.cif missing\n";
	my @lines=<LOG>;
	my @cif=<CIF>;
	&settype(@lines);
	&setxyz(@lines);
	$natom=@x;
	my $re=&min_r;
	my $R=&max_rij;
	my $dp=exp(-2*$re/$R);
	my $d=&density(@cif);
	next if $d eq undef;
	my $devi = 100*($ro/$d-1);
	printf OUT "%10s%12s%8.4f%8.4f%6.2f%6.1f%4d %10.4f  %8.4f\n", $cif,$mol, $d, $ro,$re, $R, $natom, $dp,  $devi;
	printf  "%10s%12s%8.4f%8.4f%6.2f%6.1f%4d %10.4f  %8.4f\n", $cif,$mol, $d, $ro,$re, $R, $natom, $dp,  $devi;
	close LOG;
	close CIF;
}
close CSV;


sub chkerror{
	system "rm tmp*";
	my $cif=shift @_;
	my $flag;
	system "find ./ -name $cif.cif > tmp1";
	system "find ./ -name $cif.log > tmp2";
	open TMP1, "< tmp1";
	my @tmp1=<TMP1>;
	my $tmp1 = shift @tmp1;
	close TMP1;
	open TMP2, "< tmp2";
	my @tmp2=<TMP2>;
	my $tmp2 = shift @tmp2;
	close TMP2;
	if ($tmp1 eq undef) {$flag=1;}
	if ($tmp2 eq undef) {$flag=1;}
	return $flag;
}


sub density{
	my @cif=@_;
	my $len=@cif;
	my $density;
	for (0 .. $len - 1){
		my @line=split /\s+/, $cif[$_];
		shift @line if $line[0] eq '';
		if ($line[0] eq '_exptl_crystal_density_diffrn'){
			$density=$line[1];
			last;
		}
	}
	return $density;
}


sub setxyz{
	my @lines=@_;
	my $len=@lines;
	my $loc=0;
	for (0 .. $len-1){
		$loc=$_;
		my $line=$lines[$loc];
		$line=~s/^\s+//;
		$line=~s/\s+$//;
		last if ($line=~/Standard orientation:/);
	}
	for ($loc+5 .. $len-1){
		my $line = $lines[$_];
		$line=~s/^\s+//;
		$line=~s/\s+$//;
		last if $line=~/----/;
		my @data=split /\s+/, $line;
		next if $data[0]!~/[0-9]/;
		push @x, $data[3];
		push @y, $data[4];
		push @z, $data[5];
	}
}


sub settype{
	my @lines=@_;
	my $len=@lines;
	my $loc=0;
	for (0 .. $len-1){
		$loc=$_;
		my $line=$lines[$loc];
		$line=~s/^\s+//;
		$line=~s/\s+$//;
		last if ($line=~/Symbolic Z-Matrix:/);
	}
	for ($loc+1 .. $len-1){
		my $line = $lines[$_];
		$line=~s/^\s+//;
		$line=~s/\s+$//;
		last if $line eq undef;
		my @data=split /\s+/, $line;
		push @type, $data[0];
	}
}


sub get_data{
	my $name=shift @_;
	$name=~s/\.log$//;
	open TXT, "< density7.10.txt" or die "****Error!!! DP.txt missing\n";
	my @txt=<TXT>;
	my $len=@txt;
	my $devi=-100;
	my $nj, $oj, $hj, $mass;
	for (0 .. $len-1){
		my $line=$txt[$_];
		$line=~s/^\s+//;
		$line=~s/\s+$//;
		my @data=split /\s+/, $line;
		if ($data[1] eq $name){
			$devi=100*($data[6]-$data[4])/$data[4];
			$mass=$data[13];
			$hj=$data[17];
			$oj=$data[18];
			$nj=$data[19];
			last;
		}
	}
	close TXT;
	return $devi, $hj, $oj, $nj, $mass;
}


sub max_rij{
	my $max_rij=0;
	for (0 .. $natom-1){
		my $i=$_;
		for ($i+1 .. $natom-1){
			my $j=$_;
			my $r_ij=sqrt(($x[$i]-$x[$j])**2+($y[$i]-$y[$j])**2+($z[$i]-$z[$j])**2);
			$max_rij=$r_ij if $max_rij<$r_ij;
		}
	}
	return $max_rij;
}


sub min_r{
	my @score;
	my $iter=0;
	my @seed=qw /1 1 1 1/;
	my @sw;
	my @pi;
	my @pg;
	my @velocity;
	my $loop=@seed;
	for (0 .. 40){
		my $i=$_;
		for (0 .. $loop-1){
			my $j=$_;
			$sw[$i*$loop+$j]=$seed[$j]+10000*sin(rand(10000));
		}
		$score[$i]=9999999;
	}
	$best=9999999;
	$oldbest=9999;
	while($iter<10){
		my @new_sw;
		my $refresh=0;
		for (0 .. 40){
			my $i=$_;
			my $sum=0;
			my $score;
			my $a=$sw[$loop*$i];
			my $b=$sw[$loop*$i+1];
			my $c=$sw[$loop*$i+2];
			my $d=$sw[$loop*$i+3];
			for (0 .. $natom-1){
				$sum=$sum+($a*$x[$_]+$b*$y[$_]+$c*$z[$_]+$d)**2/($a**2+$b**2+$c**2);
			}
			$score = sqrt($sum/$natom);
			if ($score < $score[$i]){
				$score[$i]=$score;
				$pi[$loop*$i]=$a;
				$pi[$loop*$i+1]=$b;
				$pi[$loop*$i+2]=$c;
				$pi[$loop*$i+3]=$d;
			}
			if ($score<$best){
				$oldbest=$best;
				$best=$score;
				$pg[0]=$a;
				$pg[1]=$b;
				$pg[2]=$c;
				$pg[3]=$d;
				$iter=0;
				$refresh=1;
			}
			for (0 .. 100){
				my $i=$_;
				for (0 .. $loop-1){
					$velocity[$i*$loop+$_]=0.9*$velocity[$i*$loop+$_]+2*rand(1)*($pg[$_]-$sw[$i*$loop+$_])+2*rand(1)*($pi[$i*$loop+$_]-$sw[$i*$loop+$_]);
					$new_sw[$i*$loop+$_]=$sw[$i*$loop+$_]+$velocity[$i*$loop+$_];
				}
			}
			@sw=@new_sw;
		}
		++$iter if ($refresh=0);
	}
	return $best;
}